from django import forms
from apps.balancer.models import ArbiterNode, ArbiterTOTP
from apps.users.models import User
from django.conf import settings
from django.contrib.auth.forms import AuthenticationForm, PasswordChangeForm as PasswordChangeFormBase, \
    SetPasswordForm as SetPasswordFormBase


class NiceViewMixin():
    def __init__(self, *args, **kwargs):
        super(NiceViewMixin, self).__init__(*args, **kwargs)
        for visible in self.visible_fields():
            visible.field.widget.attrs['class'] = 'form-control'
            if isinstance(visible.field.widget, forms.widgets.CheckboxInput):
                visible.field.widget.attrs['class'] = "checkbox-field"


class SetPasswordForm(NiceViewMixin, SetPasswordFormBase):
    pass


class PasswordChangeForm(NiceViewMixin, PasswordChangeFormBase):
    pass


class CreateManagerForm(NiceViewMixin, forms.Form):
    first_name = forms.CharField(max_length=100, required=False)
    last_name = forms.CharField(max_length=100, required=False)
    email = forms.CharField(max_length=100)
    password = forms.CharField(max_length=100, required=False, widget=forms.PasswordInput())

    notes = 'Leave "Password" empty for auto-generation'

    def clean_email(self):
        email = self.cleaned_data.get('email', '').lower()
        if User.objects.filter(email=email).exists():
            raise forms.ValidationError("This email is already used")
        return email


class CreateHostForm(NiceViewMixin, forms.Form):
    host_address = forms.CharField(required=False)

    notes = 'You can fill "Host address" later'

    def clean_host_address(self):
        return self.cleaned_data.get('host_address', 'unset')


class CreateNodeForm(NiceViewMixin, forms.Form):
    name = forms.CharField(max_length=100)
    description = forms.CharField(required=False, widget=forms.Textarea(attrs={"rows":"5"}))

    def clean_description(self):
        return self.cleaned_data.get('description', '-')

    def clean_name(self):
        name = self.cleaned_data.get('name', '').lower()
        if not name.isalnum() or name[0].isdigit():
            raise forms.ValidationError("The name must be alphanumeric")
        if ArbiterNode.objects.filter(name=name).exists():
            raise forms.ValidationError("The name is already used")
        return name


class CredentailsNodeForm(NiceViewMixin, forms.Form):
    postgres_host = forms.CharField(max_length=100)
    postgres_port = forms.IntegerField(required=False)
    postgres_dbname = forms.CharField(max_length=100)
    postgres_user = forms.CharField(max_length=100)
    postgres_password = forms.CharField(max_length=100, widget=forms.PasswordInput(render_value=True))

    redis_host = forms.CharField(max_length=100)
    redis_ssl = forms.BooleanField(required=False)
    redis_port = forms.IntegerField(required=False)
    redis_dbnumber = forms.IntegerField()
    redis_user = forms.CharField(max_length=100, required=False)
    redis_password = forms.CharField(max_length=100, widget=forms.PasswordInput(render_value=True), required=False)


class ForNewNodeForm(NiceViewMixin, forms.Form):
    pass


class TotpSkipForm(NiceViewMixin, forms.Form):
    otpbase = forms.CharField(widget=forms.HiddenInput())

    def __init__(self, user, *args, **kwargs):
        super(TotpSkipForm, self).__init__(*args, **kwargs)


class TotpSkip2Form(NiceViewMixin, forms.Form):

    def __init__(self, user, *args, **kwargs):
        super(TotpSkipForm, self).__init__(*args, **kwargs)


class TotpAskForm(NiceViewMixin, forms.Form):
    otpbase = forms.CharField(widget=forms.HiddenInput(), required=False)
    otp = forms.CharField(max_length=100, widget=forms.PasswordInput, required=True)

    def __init__(self, user, *args, **kwargs):
        super(TotpAskForm, self).__init__(*args, **kwargs)
        self._user = user


    def clean(self):
        otp = self.cleaned_data.get("otp")
        otpbase = self.cleaned_data.get("otpbase")
        if otpbase:
            if not ArbiterTOTP.validate_otp_static(otp, otpbase):
                raise forms.ValidationError("Wrong OTP")
        elif self._user:
            if not self._user.atotp.first() or not self._user.atotp.first().validate_otp(otp):
                raise forms.ValidationError("Wrong OTP")
        else:
            raise forms.ValidationError("Wrong OTP")

        return self.cleaned_data


class WebsitesStartClearForm(NiceViewMixin, forms.Form):
    notes = "You cannot directly transfer this Website to the selected Node, because this Node contains outdated version of this Website (the Node previously served this Website). To transfer the Website to this Node, you need to clear outdated data on the Node."


class WebsitesStartTransferForm(NiceViewMixin, forms.Form):
    # transfer_to = forms.ChoiceField(choices=[])
    
    def __init__(self, params, *args, **kwargs):
        self.request_params = params
        transfer_to_vals = params.get('transfer_to_vals')
        super(WebsitesStartTransferForm, self).__init__(*args, **kwargs)
        self.fields["transfer_to"] = forms.ChoiceField(
            choices=transfer_to_vals,
            widget=forms.Select(choices=transfer_to_vals, attrs={'class':'form-control'})
        )

    def clean_transfer_to(self):
        transfer_to = self.cleaned_data.get('transfer_to', '')
        if not transfer_to:
           raise forms.ValidationError("Target node is required")
        for _q, _ in self.request_params.get('transfer_to_vals', []):
            if f"{transfer_to}" == f"{_q}":
                return transfer_to
        raise forms.ValidationError("Specified target cannot be used")

class DeleteNodeForm(NiceViewMixin, forms.Form):
    name = forms.CharField(max_length=100)

    notes = 'Fill "name" of current node for deleting'

    def __init__(self, params, *args, **kwargs):
        self.request_params = params
        super(DeleteNodeForm, self).__init__(*args, **kwargs)

    def clean_name(self):
        name = self.cleaned_data.get('name', '').lower()
        node = ArbiterNode.objects.get(pk=self.request_params.get('pk'))
        if node.name != name:
            raise forms.ValidationError("Wrond node 'name' value")
        return name





class EmailHostForm(NiceViewMixin, forms.Form):
    email_address = forms.EmailField(max_length=150)


class DeleteHostForm(NiceViewMixin, forms.Form):
    host_address = forms.CharField(max_length=100)

    notes = 'Fill "host_address" of current host for deleting'

    def __init__(self, params, *args, **kwargs):
        self.request_params = params
        super(DeleteHostForm, self).__init__(*args, **kwargs)

    def clean_name(self):
        host_address = self.cleaned_data.get('host_address', '').lower()
        host = ArbiterNodeHost.objects.get(pk=self.request_params.get('pk'))
        if host.host_address != host_address:
            raise forms.ValidationError("Wrond host 'host_address' value")
        return host_address


class DeleteUserForm(NiceViewMixin, forms.Form):
    email = forms.CharField(max_length=100)

    notes = 'Fill "email" of user for deleting'

    def __init__(self, manager, *args, **kwargs):
        self.request_manager = manager
        super(DeleteUserForm, self).__init__(*args, **kwargs)

    def clean_name(self):
        email = self.cleaned_data.get('email', '').lower()
        if self.request_manager.email != email:
            raise forms.ValidationError("Wrond 'email' value")
        return email


class ArbiterAuthenticationForm(NiceViewMixin, AuthenticationForm):
    totp = forms.CharField(required=False, widget=forms.PasswordInput(), label="OTP")

    def confirm_login_allowed(self, user):
        if not user.is_superuser:
            raise forms.ValidationError("This account cannot be used here.")
        super(ArbiterAuthenticationForm, self).confirm_login_allowed(user)

        if user.atotp.exists():
            code = self.cleaned_data.get('totp', None)
            if not code or len(code) != 6 or not code.isdigit():
                raise forms.ValidationError("TOTP code is requied (6 digit)")
            for totp in user.atotp.all():
                if totp.validate_otp(code):
                    return
            raise forms.ValidationError("Wrong TOTP code")



class AppearanceForm(NiceViewMixin, forms.Form):

    use_files = True
    opts = [(d, d,) for d in settings.ENV_ALL_DOMAINS]

    title = forms.CharField(max_length=150, required=False, label='Title')
    domain = forms.ChoiceField(choices=opts, label='Main domain')
    c_text = forms.CharField(max_length=150, required=False, label='Copyright text')
    c_url = forms.URLField(max_length=150, required=False, label='Copyright link')
    base_icon = forms.FileField(required=False)

    labels = {
        'title': 'Application',
        'c_text': 'Widget',
    }
    preview = None
    preview_txt = None


    def __init__(self, *args, **kwargs):
        # self.request_params = params
        # kwargs['initial'] = params
        if preview := kwargs.get('initial', {}).get('base_icon'):
            setattr(self, 'preview', preview)
            setattr(self, 'preview_txt', 'Current global icon')
        super(AppearanceForm, self).__init__(*args, **kwargs)
